@@ -23,8 +23,10 @@
 
 #include <cuda/__device/compute_capability.h>
 #include <cuda/__fwd/devices.h>
+#include <cuda/std/__fwd/format.h>
 #include <cuda/std/__type_traits/always_false.h>
 #include <cuda/std/__utility/to_underlying.h>
+#include <cuda/std/array>
 
 #include <cuda/std/__cccl/prologue.h>
 
@@ -59,6 +61,21 @@ enum class arch_id : int
   sm_121a = 121 * __arch_specific_id_multiplier,
 };
 
+[[nodiscard]] _CCCL_API constexpr auto __all_arch_ids() noexcept
+{
+  return ::cuda::std::array{
+    arch_id::sm_60,   arch_id::sm_61,   arch_id::sm_62,   arch_id::sm_70,   arch_id::sm_75,  arch_id::sm_80,
+    arch_id::sm_86,   arch_id::sm_87,   arch_id::sm_88,   arch_id::sm_89,   arch_id::sm_90,  arch_id::sm_100,
+    arch_id::sm_103,  arch_id::sm_110,  arch_id::sm_120,  arch_id::sm_121,  arch_id::sm_90a, arch_id::sm_100a,
+    arch_id::sm_103a, arch_id::sm_110a, arch_id::sm_120a, arch_id::sm_121a,
+  };
+}
+
+[[nodiscard]] _CCCL_API constexpr bool __is_specific_arch(arch_id __arch) noexcept
+{
+  return ::cuda::std::to_underlying(__arch) > __arch_specific_id_multiplier;
+}
+
 [[nodiscard]] _CCCL_API constexpr bool __has_known_arch(compute_capability __cc) noexcept
 {
   switch (__cc.get())
@@ -126,6 +143,40 @@ enum class arch_id : int
 
 _CCCL_END_NAMESPACE_CUDA
 
+#if __cpp_lib_format >= 201907L
+_CCCL_BEGIN_NAMESPACE_STD
+
+template <class _CharT>
+struct formatter<::cuda::arch_id, _CharT> : private formatter<::cuda::compute_capability, _CharT>
+{
+  template <class _ParseCtx>
+  _CCCL_HOST_API constexpr auto parse(_ParseCtx& __ctx)
+  {
+    return __ctx.begin();
+  }
+
+  template <class _FmtCtx>
+  _CCCL_HOST_API auto format(const ::cuda::arch_id& __arch, _FmtCtx& __ctx) const
+  {
+    auto __it = __ctx.out();
+    *__it++   = _CharT{'s'};
+    *__it++   = _CharT{'m'};
+    *__it++   = _CharT{'_'};
+    __ctx.advance_to(__it);
+    __it = formatter<::cuda::compute_capability, _CharT>::format(::cuda::compute_capability{__arch}, __ctx);
+    if (::cuda::__is_specific_arch(__arch))
+    {
+      *__it++ = _CharT{'a'};
+    }
+    return __it;
+  }
+};
+
+_CCCL_END_NAMESPACE_STD
+#endif // __cpp_lib_format >= 201907L
+
+// todo: specialize cuda::std::formatter for cuda::arch_id
+
 #if _CCCL_CUDA_COMPILATION()
 
 _CCCL_BEGIN_NAMESPACE_CUDA_DEVICE
